package com.sun.jndi.url.ldap;

import java.util.StringTokenizer;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.Attributes;
import javax.naming.NameParser;
import javax.naming.Context;
import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.InvalidNameException;
import javax.naming.CompositeName;
import com.sun.jndi.ldap.LdapURL;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.spi.ResolveResult;
import java.util.Hashtable;
import com.sun.jndi.toolkit.url.GenericURLDirContext;

public final class ldapURLContext extends GenericURLDirContext
{
    ldapURLContext(Hashtable<?, ?> env) {
        super(env);
    }
    
    @Override
    protected ResolveResult getRootURLContext(String s, Hashtable<?, ?> hashtable) throws NamingException {
        return ldapURLContextFactory.getUsingURLIgnoreRootDN(s, hashtable);
    }
    
    @Override
    protected Name getURLSuffix(String s, String url) throws NamingException {
        LdapURL ldapURL = new LdapURL(url);
        String s2 = (ldapURL.getDN() != null) ? ldapURL.getDN() : "";
        CompositeName compositeName = new CompositeName();
        if (!"".equals(s2)) {
            compositeName.add(s2);
            return compositeName;
        }
        return compositeName;
    }
    
    @Override
    public Object lookup(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.lookup(explanation);
    }
    
    @Override
    public Object lookup(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.lookup(name);
    }
    
    @Override
    public void bind(String explanation, Object obj) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.bind(explanation, obj);
        return;
    }
    
    @Override
    public void bind(Name name, Object obj) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.bind(name, obj);
        return;
    }
    
    @Override
    public void rebind(String explanation, Object obj) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.rebind(explanation, obj);
        return;
    }
    
    @Override
    public void rebind(Name name, Object obj) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.rebind(name, obj);
        return;
    }
    
    @Override
    public void unbind(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.unbind(explanation);
        return;
    }
    
    @Override
    public void unbind(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.unbind(name);
        return;
    }
    
    @Override
    public void rename(String explanation, String explanation2) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        if (LdapURL.hasQueryComponents(explanation2)) {
            throw new InvalidNameException(explanation2);
        }
        super.rename(explanation, explanation2);
        return;
    }
    
    @Override
    public void rename(Name name, Name newName) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        if (LdapURL.hasQueryComponents(newName.get(0))) {
            throw new InvalidNameException(newName.toString());
        }
        super.rename(name, newName);
        return;
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.list(explanation);
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.list(name);
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.listBindings(explanation);
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.listBindings(name);
    }
    
    @Override
    public void destroySubcontext(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.destroySubcontext(explanation);
        return;
    }
    
    @Override
    public void destroySubcontext(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.destroySubcontext(name);
        return;
    }
    
    @Override
    public Context createSubcontext(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.createSubcontext(explanation);
    }
    
    @Override
    public Context createSubcontext(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.createSubcontext(name);
    }
    
    @Override
    public Object lookupLink(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.lookupLink(explanation);
    }
    
    @Override
    public Object lookupLink(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.lookupLink(name);
    }
    
    @Override
    public NameParser getNameParser(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.getNameParser(explanation);
    }
    
    @Override
    public NameParser getNameParser(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.getNameParser(name);
    }
    
    @Override
    public String composeName(String explanation, String explanation2) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        if (LdapURL.hasQueryComponents(explanation2)) {
            throw new InvalidNameException(explanation2);
        }
        return super.composeName(explanation, explanation2);
    }
    
    @Override
    public Name composeName(Name name, Name prefix) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        if (LdapURL.hasQueryComponents(prefix.get(0))) {
            throw new InvalidNameException(prefix.toString());
        }
        return super.composeName(name, prefix);
    }
    
    @Override
    public Attributes getAttributes(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.getAttributes(explanation);
    }
    
    @Override
    public Attributes getAttributes(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.getAttributes(name);
    }
    
    @Override
    public Attributes getAttributes(String explanation, String[] attrIds) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.getAttributes(explanation, attrIds);
    }
    
    @Override
    public Attributes getAttributes(Name name, String[] attrIds) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.getAttributes(name, attrIds);
    }
    
    @Override
    public void modifyAttributes(String explanation, int mod_op, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.modifyAttributes(explanation, mod_op, attrs);
        return;
    }
    
    @Override
    public void modifyAttributes(Name name, int mod_op, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.modifyAttributes(name, mod_op, attrs);
        return;
    }
    
    @Override
    public void modifyAttributes(String explanation, ModificationItem[] mods) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.modifyAttributes(explanation, mods);
        return;
    }
    
    @Override
    public void modifyAttributes(Name name, ModificationItem[] mods) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.modifyAttributes(name, mods);
        return;
    }
    
    @Override
    public void bind(String explanation, Object obj, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.bind(explanation, obj, attrs);
        return;
    }
    
    @Override
    public void bind(Name name, Object obj, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.bind(name, obj, attrs);
        return;
    }
    
    @Override
    public void rebind(String explanation, Object obj, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        super.rebind(explanation, obj, attrs);
        return;
    }
    
    @Override
    public void rebind(Name name, Object obj, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        super.rebind(name, obj, attrs);
        return;
    }
    
    @Override
    public DirContext createSubcontext(String explanation, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.createSubcontext(explanation, attrs);
    }
    
    @Override
    public DirContext createSubcontext(Name name, Attributes attrs) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.createSubcontext(name, attrs);
    }
    
    @Override
    public DirContext getSchema(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.getSchema(explanation);
    }
    
    @Override
    public DirContext getSchema(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.getSchema(name);
    }
    
    @Override
    public DirContext getSchemaClassDefinition(String explanation) throws NamingException {
        if (LdapURL.hasQueryComponents(explanation)) {
            throw new InvalidNameException(explanation);
        }
        return super.getSchemaClassDefinition(explanation);
    }
    
    @Override
    public DirContext getSchemaClassDefinition(Name name) throws NamingException {
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.getSchemaClassDefinition(name);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes matchingAttributes) throws NamingException {
        if (LdapURL.hasQueryComponents(s)) {
            return this.searchUsingURL(s);
        }
        return super.search(s, matchingAttributes);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes matchingAttributes) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), matchingAttributes);
        }
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.search(name, matchingAttributes);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes matchingAttributes, String[] attributesToReturn) throws NamingException {
        if (LdapURL.hasQueryComponents(s)) {
            return this.searchUsingURL(s);
        }
        return super.search(s, matchingAttributes, attributesToReturn);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes matchingAttributes, String[] attributesToReturn) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), matchingAttributes, attributesToReturn);
        }
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.search(name, matchingAttributes, attributesToReturn);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, String filter, SearchControls cons) throws NamingException {
        if (LdapURL.hasQueryComponents(s)) {
            return this.searchUsingURL(s);
        }
        return super.search(s, filter, cons);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String filter, SearchControls cons) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), filter, cons);
        }
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.search(name, filter, cons);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, String filterExpr, Object[] filterArgs, SearchControls cons) throws NamingException {
        if (LdapURL.hasQueryComponents(s)) {
            return this.searchUsingURL(s);
        }
        return super.search(s, filterExpr, filterArgs, cons);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String filterExpr, Object[] filterArgs, SearchControls cons) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), filterExpr, filterArgs, cons);
        }
        if (LdapURL.hasQueryComponents(name.get(0))) {
            throw new InvalidNameException(name.toString());
        }
        return super.search(name, filterExpr, filterArgs, cons);
    }
    
    private NamingEnumeration<SearchResult> searchUsingURL(String url) throws NamingException {
        LdapURL ldapURL = new LdapURL(url);
        ResolveResult rootURLContext = this.getRootURLContext(url, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.search(rootURLContext.getRemainingName(), setFilterUsingURL(ldapURL), setSearchControlsUsingURL(ldapURL));
        }
        finally {
            dirContext.close();
        }
    }
    
    private static String setFilterUsingURL(LdapURL ldapURL) {
        String filter = ldapURL.getFilter();
        if (filter == null) {
            filter = "(objectClass=*)";
            return filter;
        }
        return filter;
    }
    
    private static SearchControls setSearchControlsUsingURL(LdapURL ldapURL) {
        SearchControls searchControls = new SearchControls();
        String scope = ldapURL.getScope();
        String attributes = ldapURL.getAttributes();
        if (scope == null) {
            searchControls.setSearchScope(0);
        }
        else if (scope.equals("sub")) {
            searchControls.setSearchScope(2);
        }
        else if (scope.equals("one")) {
            searchControls.setSearchScope(1);
        }
        else if (scope.equals("base")) {
            searchControls.setSearchScope(0);
        }
        if (attributes == null) {
            searchControls.setReturningAttributes((String[])null);
            return searchControls;
        }
        StringTokenizer stringTokenizer = new StringTokenizer(attributes, ",");
        int countTokens = stringTokenizer.countTokens();
        String[] returningAttributes = new String[countTokens];
        for (int i = 0; i < countTokens; ++i) {
            returningAttributes[i] = stringTokenizer.nextToken();
        }
        searchControls.setReturningAttributes(returningAttributes);
        return searchControls;
    }
}
